indexing

	description:

		"Reader where items from a reader are transformed into something else."

	library: "Gobo Eiffel Pipework Library"
	copyright: "Copyright (c) 2003, Simon Parker and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: $"
	revision: "$Revision: $"


deferred class

	PW_TRANSFORMING_READER [OUTWARD]


inherit

	PW_SOURCE_READER [OUTWARD]
		redefine
			clear_error,
			close,
			end_of_input,
			is_closable,
			is_open_read,
			rewind,
			source
		end


feature -- Error handling

	clear_error is
			-- Reset any error codes. Called before pump starts.
		do
			precursor
			if source /= Void then
				source.clear_error
			end
		end

	copy_error_from_source is
			-- If not `source'.`is_ok', it's error code and diagnostic
			-- message is copied. Else nothing happens.
		do
			if not source.is_ok then
				diagnostic_code := source.diagnostic_code
				diagnostic_message := source.diagnostic_message
			end
		ensure
			error_copied: not source.is_ok implies
				(diagnostic_code = source.diagnostic_code and
				 diagnostic_message.is_equal (source.diagnostic_message))
			not_copied: source.is_ok implies
				diagnostic_code = old diagnostic_code
		end


feature -- Status

	end_of_input: BOOLEAN is
			-- Are there no more objects?
		do
			Result := source = Void or else source.end_of_input
		end

	is_closable: BOOLEAN is
		do
			Result := source /= Void and then source.is_closable
		end

	is_open_read: BOOLEAN is
			-- Can items be read from input stream?
		do
			Result := source /= Void and then source.is_open_read
		end


feature -- Access

	source: PW_READER [ANY]
			-- Inlet


feature -- Basic operations

	close is
		do
			if source /= Void then
				source.close
			end
		end


feature -- Reading

	rewind is
			-- Make `last_read' invalid.
		do
			source.rewind
		ensure then
			source_ok_is_something_to_read: source.is_ok = not source.end_of_input
		end

end
